/*
 * SPI2CF (c) 2006, Till Harbaum.
 * All rights reserved.
 *
 * This file is distributed under the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part SPI2CF port of the uIP TCP/IP stack.
 */

/*
  prism2.c

  Based on the prism2.c file from pascal stangs avrlib.
*/

#include "uip.h"
#include "uip_arp.h"

#include <stdio.h>
#include <stdint.h>
#include <string.h>

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "cf.h"
#include "prism2.h"

// #define DEBUG_CMD

#define MAGIC 0x4711

#ifdef PRISM2_ADHOC
#define	PRISM2_RID_CNFSSID  PRISM2_RID_CNFOWNSSID
#define PRISM2_MODE         PRISM2_MODE_IBSS
#else
#ifdef PRISM2_MANAGED
#define	PRISM2_RID_CNFSSID  PRISM2_RID_CNFDESIREDSSID
#define PRISM2_MODE         PRISM2_MODE_BSS
#else
#error Please specify PRISM2_MANAGED or PRISM2_ADHOC!
#endif
#endif

#define PRISM2_READ_WORD(a)    cf_io_read_word(a)
#define PRISM2_WRITE_WORD(a,v) cf_io_write_word(a,v)

// keep one global tx buffer
uint16_t txfid;

void prism2_enable(void) {
  uint8_t u;
  uint16_t s;
  
  printf_P(PSTR("\nPrism2 PCMCIA attribute registers:\n"));
  printf_P(PSTR("COR: 0x%02x\n"), cf_reg_read_byte(0x3e0 + PCMCIA_ATTR_COR));
  printf_P(PSTR("CSR: 0x%02x\n"), cf_reg_read_byte(0x3e0 + PCMCIA_ATTR_CSR));
  printf_P(PSTR("PRR: 0x%02x\n"), cf_reg_read_byte(0x3e0 + PCMCIA_ATTR_PRR));

  printf_P(PSTR("Enabling IO access ...\n"));
  cf_reg_write_byte(0x3e0 + PCMCIA_ATTR_COR, 
		    cf_reg_read_byte(0x3e0 + PCMCIA_ATTR_COR) | 0x41);

  printf_P(PSTR("COR: 0x%02x\n"), cf_reg_read_byte(0x3e0 + PCMCIA_ATTR_COR));

  // check for card availablity by writing and verifying one
  // of the software register
  PRISM2_WRITE_WORD(PRISM2_REG_SWSUP0, MAGIC);
  s = PRISM2_READ_WORD(PRISM2_REG_SWSUP0);
  if(s != MAGIC) {
    printf_P(PSTR("Test write failed (%x != %x)...\n"), s, MAGIC);
    for(;;);
  }
} 

void prism2_reg_dump(void) {
  printf_P(PSTR("\nPrism2 register dump:\n"));
  printf_P(PSTR("CMD     : %x\n"), PRISM2_READ_WORD(PRISM2_REG_CMD));
  printf_P(PSTR("PARAM0  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_PARAM0));
  printf_P(PSTR("PARAM1  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_PARAM1));
  printf_P(PSTR("PARAM2  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_PARAM2));
  printf_P(PSTR("STATUS  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_STATUS));
  printf_P(PSTR("RESP0   : %x\n"), PRISM2_READ_WORD(PRISM2_REG_RESP0));
  printf_P(PSTR("RESP1   : %x\n"), PRISM2_READ_WORD(PRISM2_REG_RESP1));
  printf_P(PSTR("RESP2   : %x\n"), PRISM2_READ_WORD(PRISM2_REG_RESP2));

  printf_P(PSTR("INFOFID : %x\n"), PRISM2_READ_WORD(PRISM2_REG_INFOFID));
  printf_P(PSTR("RXFID   : %x\n"), PRISM2_READ_WORD(PRISM2_REG_RXFID));
  printf_P(PSTR("ALLOCFID: %x\n"), PRISM2_READ_WORD(PRISM2_REG_ALLOCFID));
  printf_P(PSTR("TXFID   : %x\n"), PRISM2_READ_WORD(PRISM2_REG_TXFID));

  printf_P(PSTR("BAP0SEL : %x\n"), PRISM2_READ_WORD(PRISM2_REG_BAP0SEL));
  printf_P(PSTR("BAP0OFFS: %x\n"), PRISM2_READ_WORD(PRISM2_REG_BAP0OFFSET));
  printf_P(PSTR("BAP0DATA: %x\n"), PRISM2_READ_WORD(PRISM2_REG_BAP0DATA));

  printf_P(PSTR("BAP1SEL : %x\n"), PRISM2_READ_WORD(PRISM2_REG_BAP1SEL));
  printf_P(PSTR("BAP1OFFS: %x\n"), PRISM2_READ_WORD(PRISM2_REG_BAP1OFFSET));
  printf_P(PSTR("BAP1DATA: %x\n"), PRISM2_READ_WORD(PRISM2_REG_BAP1DATA));

  printf_P(PSTR("EVSTAT  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_EVSTAT));
  printf_P(PSTR("INTEN   : %x\n"), PRISM2_READ_WORD(PRISM2_REG_INTEN));
  printf_P(PSTR("EVACK   : %x\n"), PRISM2_READ_WORD(PRISM2_REG_EVACK));
  
  printf_P(PSTR("SWSUP0  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_SWSUP0));
  printf_P(PSTR("SWSUP0  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_SWSUP1));
  printf_P(PSTR("SWSUP0  : %x\n"), PRISM2_READ_WORD(PRISM2_REG_SWSUP2));
  
  printf_P(PSTR("AUXPAGE : %x\n"), PRISM2_READ_WORD(PRISM2_REG_AUXPAGE));
  printf_P(PSTR("AUXOFFS : %x\n"), PRISM2_READ_WORD(PRISM2_REG_AUXOFFSET));
  printf_P(PSTR("AUXDATA : %x\n"), PRISM2_READ_WORD(PRISM2_REG_AUXDATA));
}

uint8_t prism2_command(uint16_t cmd, uint16_t param0) {
  uint16_t result;

  // wait until card not busy
#ifdef DEBUG_CMD
  printf_P(PSTR("PRISM_CMD: Wait until card ready\n"));
#endif
  while(PRISM2_READ_WORD(PRISM2_REG_CMD) & PRISM2_CMD_BUSY);
	
#ifdef DEBUG_CMD
  printf_P(PSTR("PRISM_CMD: Issue Command = 0x%x\n"), cmd);
#endif
  PRISM2_WRITE_WORD(PRISM2_REG_PARAM0, param0);
  PRISM2_WRITE_WORD(PRISM2_REG_CMD, cmd);
  
  // wait until card not busy
#ifdef DEBUG_CMD
  printf_P(PSTR("PRISM_CMD: Wait until card ready\n"));
#endif
  while(PRISM2_READ_WORD(PRISM2_REG_CMD) & PRISM2_CMD_BUSY);
  
  // read event register - wait for command to complete
#ifdef DEBUG_CMD
  printf_P(PSTR("PRISM_CMD: Wait for command to complete\n"));
#endif
  while(!(PRISM2_READ_WORD(PRISM2_REG_EVSTAT) & PRISM2_EVENT_CMD));
  
  // read status register
  result = PRISM2_READ_WORD(PRISM2_REG_STATUS);
#ifdef DEBUG_CMD
  printf_P(PSTR("PRISM_CMD: Result  = 0x%x\n"), result>>8);
  printf_P(PSTR("PRISM_CMD: CmdCode = 0x%x\n"), result);
#endif
  
  // acknowledge event
#ifdef DEBUG_CMD
  printf_P(PSTR("PRISM_CMD: Ack command event\n"));
#endif
  PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_CMD);
  
  // return command result
  return result>>8;
}

uint8_t prism2_read_BAP0(uint16_t id, uint16_t offset, 
			 uint16_t* data, uint16_t len) {
  // wait for BAP to be ready
//  printf_P(PSTR("PRISM_BAP: Wait1\n"));
  while( PRISM2_READ_WORD(PRISM2_REG_BAP0OFFSET) & PRISM2_BAPOFFSET_BUSY);
  // set ID
  PRISM2_WRITE_WORD(PRISM2_REG_BAP0SEL, id);
  // set offset
  PRISM2_WRITE_WORD(PRISM2_REG_BAP0OFFSET, offset);
  // wait for BAP to be ready
//  printf_P(PSTR("PRISM_BAP: Wait2\n"));
  while( PRISM2_READ_WORD(PRISM2_REG_BAP0OFFSET) & PRISM2_BAPOFFSET_BUSY);
  // check for error
  if(PRISM2_READ_WORD(PRISM2_REG_BAP0OFFSET) & PRISM2_BAPOFFSET_ERROR)
    return -1;
  // read data
//  printf_P(PSTR("PRISM_BAP: Read\n"));

#if 0
  while(len--)
    *data++ = PRISM2_READ_WORD(PRISM2_REG_BAP0DATA);
#endif

  cf_io_read(PRISM2_REG_BAP0DATA, data, len);

  // return success
  return 0;
}

uint8_t prism2_write_BAP0(uint16_t id, uint16_t offset, 
			  uint16_t* data, uint16_t len) {

  // wait for BAP to be ready
//  printf_P(PSTR("PRISM_BAP: Wait1\n"));
  while(PRISM2_READ_WORD(PRISM2_REG_BAP0OFFSET) & PRISM2_BAPOFFSET_BUSY);
  // set ID
  PRISM2_WRITE_WORD(PRISM2_REG_BAP0SEL, id);
  // set offset
  PRISM2_WRITE_WORD(PRISM2_REG_BAP0OFFSET, offset);
  // wait for BAP to be ready
//  printf_P(PSTR("PRISM_BAP: Wait2\n"));
  while( PRISM2_READ_WORD(PRISM2_REG_BAP0OFFSET) & PRISM2_BAPOFFSET_BUSY);
  // check for error
  if(PRISM2_READ_WORD(PRISM2_REG_BAP0OFFSET) & PRISM2_BAPOFFSET_ERROR)
    return -1;
  // write data
//  printf_P(PSTR("PRISM_BAP: Write\n"));
#if 0
  while(len--)
    PRISM2_WRITE_WORD(PRISM2_REG_BAP0DATA, *data++);
#endif

  cf_io_write(PRISM2_REG_BAP0DATA, data, len);

  // return success
  return 0;
}

uint8_t prism2_read_RID(uint16_t id, uint16_t offset, 
			uint16_t* data, uint16_t len) {
  prism2_command(PRISM2_CMD_ACCESS_RD, id);
  return prism2_read_BAP0(id, offset, data, len);
}

uint8_t prism2_write_RID(uint16_t offset, uint16_t* data) {
  uint8_t result;
  result = prism2_write_BAP0(data[1], offset, data, data[0]+1);
  prism2_command(PRISM2_CMD_ACCESS_WR, data[1]);
  return result;
}

uint8_t prism2_write_RID2(uint16_t id, uint16_t val) {
  uint16_t buffer[3] = { 2, id, val };
  uint8_t result;
  result = prism2_write_BAP0(id, 0, buffer, 3);
  prism2_command(PRISM2_CMD_ACCESS_WR, id);
  return result;
}

void prism2_set_SSID(uint8_t *ssid) {
  uint16_t buffer[12];

  // prepare buffer for SSID write
  buffer[0] = 0x0012;
  buffer[1] = PRISM2_RID_CNFSSID;
  buffer[2] = strlen(ssid);
  // copy ssid string to buffer
  strcpy((uint8_t*)&buffer[3], ssid);
  // write SSID
  prism2_write_RID(0, buffer);
}

void prism2_get_MAC(uint8_t *mac) {
  uint16_t buffer[5];

  // read MAC address register
  prism2_read_RID(PRISM2_RID_CNFOWNMACADDR, 0, buffer, 5);
  
  *mac++ = buffer[2];
  *mac++ = buffer[2]>>8;
  *mac++ = buffer[3];
  *mac++ = buffer[3]>>8;
  *mac++ = buffer[4];
  *mac++ = buffer[4]>>8;
}

#ifdef PRISM2_WEPKEY
#define PRISM2_WEPCMDLEN  ((PRISM2_WEPLEN+3)/2)

#if ((PRISM2_WEPLEN != 5) && (PRISM2_WEPLEN != 13))
#error Illegal PRISM2_WEPLEN key size!
#endif

void prism2_set_WEPkey(void) {
  uint8_t wepkey[PRISM2_WEPLEN+1] = PRISM2_WEPKEY;
  uint16_t buffer[PRISM2_WEPCMDLEN+1];

  printf_P(PSTR("Enabling %d bit WEP encryption\n"), 8*PRISM2_WEPLEN);

  // prepare buffer for key write
  buffer[0] = PRISM2_WEPCMDLEN;
  buffer[1] = PRISM2_RID_CNFWEPDEFAULTKEY0;
  // copy key string to buffer
  memcpy((uint8_t*)&buffer[2], wepkey, PRISM2_WEPLEN+1);
  // write default key 0
  prism2_write_RID(0, buffer);

  // activate first wep key
  prism2_write_RID2(PRISM2_RID_CNFWEPDEFAULTKEYID, 0);

  // set WEP active
  prism2_write_RID2(PRISM2_RID_CNFWEPFLAGS, 1);

  // configure authentication mode
  prism2_write_RID2(PRISM2_RID_CNFAUTHENTICATION, 1);
}
#endif

void prism2_init(void) {
  uint8_t result;
  uint16_t buffer[20];

  printf_P(PSTR("\nPrism2 Init:\n"));

  // first init easily fails ...
  if( (result = prism2_command(PRISM2_CMD_INIT,0)) ) {
    if( (result = prism2_command(PRISM2_CMD_INIT,0)) ) {
      printf_P(PSTR("Initialization failed: %x\n"), result);
      for(;;);
    }
  }

  printf_P(PSTR("Prism2 Initialized\n"));

  prism2_get_MAC(uip_ethaddr.addr);
  printf_P(PSTR("MAC: %02x:%02x:%02x:%02x:%02x:%02x\n"),
	   uip_ethaddr.addr[0], uip_ethaddr.addr[1], 
	   uip_ethaddr.addr[2], uip_ethaddr.addr[3], 
	   uip_ethaddr.addr[4], uip_ethaddr.addr[5]);

  printf_P(PSTR("Setting max pkt size: %d\n"), PRISM2_MAX_PKT_SIZE);

  // set max packet size
  prism2_write_RID2(PRISM2_RID_CNFMAXDATALEN, PRISM2_MAX_PKT_SIZE);
  
  // set operating mode / port type
  prism2_write_RID2(PRISM2_RID_CNFPORTTYPE, PRISM2_MODE);
  
  // set SSID
  prism2_set_SSID(PRISM2_SSID);

#ifdef PRISM2_CHANNEL
  // set channel
  prism2_write_RID2(PRISM2_RID_CNFOWNCHANNEL, PRISM2_CHANNEL);
#endif

#ifdef PRISM2_WEPKEY  
  prism2_set_WEPkey();
#endif

  // enable the interface
  prism2_command(PRISM2_CMD_ENABLE_MAC0, 0);

  // request free buffer space to store outgoing frame
  prism2_command(PRISM2_CMD_ALLOC, PRISM2_MAX_PKT_SIZE);

  // wait for buffer to be allocated
  while(!(PRISM2_READ_WORD(PRISM2_REG_EVSTAT) & PRISM2_EVENT_ALLOC) );

  // get the buffer FID
  txfid = PRISM2_READ_WORD(PRISM2_REG_ALLOCFID);
}


void prism2_event_check(void) {
  uint16_t evstat_data;

  evstat_data = PRISM2_READ_WORD(PRISM2_REG_EVSTAT);
	
  if(evstat_data & PRISM2_EVENT_TX) {
    printf_P(PSTR("Event: TX\n"));
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK,PRISM2_EVENT_TX);
  }

  if(evstat_data & PRISM2_EVENT_TXEXEC) {
    printf_P(PSTR("Event: TXEXEC\n"));
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK,PRISM2_EVENT_TXEXEC);
  }
  
  if(evstat_data & PRISM2_EVENT_ALLOC) {
    printf_P(PSTR("Event: ALLOC\n"));
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_ALLOC);
  }
  
  if(evstat_data & PRISM2_EVENT_CMD) {
    printf_P(PSTR("Event: CMD\n"));
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_CMD);
  }
  
  if(evstat_data & PRISM2_EVENT_INFO) {
    printf_P(PSTR("Event: INFO\n"));
    PRISM2_READ_WORD(PRISM2_REG_INFOFID);
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_INFO);
  }
  
  if(evstat_data & PRISM2_EVENT_INFDROP) {
    printf_P(PSTR("Event: INFDROP\n"));
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_INFDROP);
  }
  
  if(evstat_data & PRISM2_EVENT_WTERR) {
    printf_P(PSTR("Event: WTERR\n"));
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_WTERR);
  }
}

uint16_t prism2_poll(uint8_t *packet, uint16_t maxlen) {
  uint16_t rxfid = 0;
  uint16_t packetLength = 0;

  // check if packets have been received
  if(PRISM2_READ_WORD(PRISM2_REG_EVSTAT) & PRISM2_EVENT_RX) {
    // we have a receive event
    // get RxFID
    rxfid = PRISM2_READ_WORD(PRISM2_REG_RXFID);
    // read the packet length
    prism2_read_BAP0(rxfid, 44, &packetLength, 1);
  }

  // if there's no packet or an error - exit
  if( !packetLength ) 
    return 0;
  
  // drop anything too big for the buffer
  if( packetLength+6 > maxlen ) {
    // ACK the receive event to finish up
    PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_RX);
    printf_P(PSTR("prism2_poll: packet to long (%d)\n"), packetLength);
    return 0;
  }
  
  // read ethernet addresses
  prism2_read_BAP0(rxfid, 2*23, (uint16_t*)&packet[0], 6);
  // and payload
  prism2_read_BAP0(rxfid, 2*33, (uint16_t*)&packet[12], (packetLength+6-1)>>1);
  // ACK the receive event to finish up
  PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_RX);
  
  return packetLength+6;
}

void prism2_setup_tx_header(uint16_t* header) {
  uint16_t i;
  
  // clear out header
  for(i=0;i<22;i++)
    header[i] = 0x00;

  // set TxRate and retry count
  header[5] = (0<<8) | 0;
  // 0x00 = automatic selection
  // 0x0A = 10  = 1.0Mbit/s
  // 0x14 = 20  = 2.0Mbit/s
  // 0x37 = 55  = 5.5Mbit/s
  // 0x6E = 110 = 11 Mbit/s
  
  // set TxControl
  header[6] = 0x0004;
  
  // write length
  // (not really needed since card will pull info from 802.3 header)
  //header[22] = len+8;
  
  // fill in 802.3 header fields
  header[30] = 0xAAAA;
  header[31] = 0x0003;
  header[32] = 0x0000;
  
  // src mac address @ byte offset 52
}

void prism2_send(uint8_t* packet, uint16_t len) {
  uint16_t TxHeader[34];
  uint16_t i;
  uint8_t stat;

  // ACK the alloc event
  PRISM2_WRITE_WORD(PRISM2_REG_EVACK, PRISM2_EVENT_ALLOC);

  // adjust packet length because MAC addresses and type
  // will be written seperately from packet payload
  len -= 14;
  
  // write the outgoing frame to BAP
  // begin with control structure
  prism2_setup_tx_header(TxHeader);
  
  // write dest and src MAC addresses
  for(i=0;i<6;++i)
    TxHeader[23+i] = packet[i*2+1]<<8 | packet[i*2];

  // write length
  TxHeader[29] = htons(len+8);
  // write type
  TxHeader[33] = packet[13]<<8 | packet[12];
  
  // write Tx header out to BAP
  prism2_write_BAP0(txfid, 0, TxHeader, 34);
  // write packet out to BAP
  prism2_write_BAP0(txfid, 68, (uint16_t*)&packet[14], (len+1)>>1);
  // issue transmit command
  stat = prism2_command(PRISM2_CMD_TX_RECL, txfid);
  if(stat)
    printf_P(PSTR("Transmit failed: 0x%x\n"), stat);
  // do cleanup
  prism2_event_check();
}

