/*
 * SPI2CF (c) 2006, Till Harbaum.
 * All rights reserved.
 *
 * This file is distributed under the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part SPI2CF port of the uIP TCP/IP stack.
 */

#include "uip-conf.h"

#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>

#include "spi.h"

void spi_init(void) {
  // SS, SCK, MOSI -> output
  DDRB |= _BV(4) | _BV(5) | _BV(7);
  // MISO -> input \w pullup
  DDRB &= ~_BV(6);
  PORTB |= _BV(6);

  PORTB &= ~_BV(7);  // set SCK lo
  PORTB |= _BV(6);   // enable MISO pullup

  // reset spi shifter
  PORTB |=  _BV(4);  // set SS high
  PORTB &= ~_BV(4);  // set SS low

  // no irq, lsb first, master, enable, edge end, falling edge
  // xtal/4 (4MHz) (xtal/16 with spr0 set)
  SPCR = _BV(MSTR) | _BV(SPE) /* | _BV(SPR0) */;
//   SPSR |= _BV(SPI2X);
}

void spi_write_word(unsigned short word) {
  unsigned char *p = (unsigned char*)&word;

  SPDR = *p++;
  while (!(SPSR & _BV(SPIF)));
  SPDR = *p++;
  while (!(SPSR & _BV(SPIF)));
}

unsigned short spi_read_word(void) {
  unsigned short result;
  unsigned char *p = (unsigned char*)&result;

  /* write dummy byte */
  SPDR = 0xFF;
  while (!(SPSR & _BV(SPIF)));
  *p++ = SPDR;

  /* write dummy byte */
  SPDR = 0xFF;
  while (!(SPSR & _BV(SPIF)));
  *p++ = SPDR;

  return result;
}

unsigned char spi_cycle_byte(unsigned char inbyte) {
  SPDR = inbyte;
  while (!(SPSR & _BV(SPIF)));
  return SPDR;
}

void spi_start(void) {
  // reset byte counter
  PORTB |=  _BV(4);  // set SS high
  PORTB &= ~_BV(4);  // set SS low
}
