/*
 * SPI2CF (c) 2006, Till Harbaum.
 * All rights reserved.
 *
 * This file is distributed under the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part SPI2CF port of the uIP TCP/IP stack.
 */

#include "uip.h"

#include <stdio.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#define UART_REG   (((F_CPU / BITRATE)-8)/16)

static int uart_putchar(char c, FILE *stream) {
  if (c == '\n')
    uart_putchar('\r', stream);
  loop_until_bit_is_set(UCSRA, UDRE);
  UDR = c;
  return 0;
}

static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL,
					 _FDEV_SETUP_WRITE);

void uart_init(void) {
  UBRRL = UART_REG;
  UBRRH = 0;
  
  UCSRA = 0;
  UCSRB = _BV(TXEN);                  // enable transmitter
  UCSRC = _BV(URSEL) | (3 << UCSZ0);  // 8n1

  stdout = &mystdout;
}

void hexdump(void *data, int size) {
  int i,n = 0, b2c;
  unsigned char *ptr = data;

  if(!size) return;

//  printf_P(PSTR("---- dump %d bytes\n"), size);

  while(size>0) {
    printf_P(PSTR("%04x: "), n);

    b2c = (size>16)?16:size;

    for(i=0;i<b2c;i++)
      printf_P(PSTR("%02x "), 0xff & ptr[i]);

    printf_P(PSTR("  "));

    for(i=0;i<(16-b2c);i++)
      printf_P(PSTR("   "));

    for(i=0;i<b2c;i++)
      printf_P(PSTR("%c"), isprint(ptr[i])?ptr[i]:'.');

    printf_P(PSTR("\n"));

    ptr  += b2c;
    size -= b2c;
    n    += b2c;
 }
}
